﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace AitP
{
	public partial class Main : Form
	{
		private bool _running = false;
		private System.Threading.Thread _t = null;

		public Main()
		{
			InitializeComponent();
		}

		private void btnStart_Click(object sender, EventArgs e)
		{
			lst.Items.Clear();
			int maxPlayers;
			int iterations;
			int assassinLynchDay = 0;
			if (!int.TryParse(txtPlayers.Text, out maxPlayers) || maxPlayers < 5)
			{
				MessageBox.Show("Invalid # of max players.");
				return;
			}
			else if (!int.TryParse(txtIterations.Text, out iterations) || iterations < 1)
			{
				MessageBox.Show("Invalid # of iterations.");
				return;
			}
			else if (!string.IsNullOrEmpty(txtAssassinLynchDay.Text) && (!int.TryParse(txtAssassinLynchDay.Text, out assassinLynchDay) || assassinLynchDay < 0))
			{
				MessageBox.Show("Invalid Assassin lynch Day.");
				return;
			}

			txtPlayers.Enabled = txtIterations.Enabled = txtAssassinLynchDay.Enabled = btnStart.Enabled = false;
			btnStop.Enabled = true;
			_running = true;
			prg.Value = 0;
			prg.Maximum = iterations;
			AitPWorker worker = new AitPWorker(this, maxPlayers, iterations, assassinLynchDay);
			_t = new System.Threading.Thread(worker.Run);
			_t.Start();
			tmr.Enabled = true;
		}

		private void btnStop_Click(object sender, EventArgs e)
		{
			tmr.Enabled = btnStop.Enabled = false;
			_running = false;
			if (_t != null) _t.Join(30000);
			txtPlayers.Enabled = txtIterations.Enabled = txtAssassinLynchDay.Enabled = btnStart.Enabled = true;
		}

		public bool IsRunning { get { return _running; } }

		delegate void LogDelegate(string message);
		delegate void IncrementDelegate();

		public void Log(string message)
		{
			if (InvokeRequired)
			{
				Invoke(new LogDelegate(Log), message);
			}
			else
			{
				while (lst.Items.Count > 500) lst.Items.RemoveAt(0);
				lst.Items.Add(message);
				lst.SelectedIndex = lst.Items.Count - 1;
			}
		}

		public void Increment()
		{
			if (InvokeRequired)
			{
				Invoke(new IncrementDelegate(Increment));
			}
			else
			{
				prg.Value = prg.Value + 1;
			}
		}

		private void tmr_Tick(object sender, EventArgs e)
		{
			if (_t != null && !_t.IsAlive) btnStop_Click(null, null);
		}
	}
}
